import sys

from PySide6.QtWidgets import QApplication

from orso_sample_builder.main_window import MainWindow


def main():
    app = QApplication(sys.argv)

    window = MainWindow()
    window.show()

    sys.exit(app.exec())


if __name__ == "__main__":
    main()



import os

from PySide6.QtWidgets import QVBoxLayout, QWidget

os.environ["QT_API"] = "PySide6"

from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas  # noqa: E402
from matplotlib.backends.backend_qtagg import NavigationToolbar2QT as NavigationToolbar  # noqa: E402
from matplotlib.figure import Figure  # noqa: E402


class MplCanvas(QWidget):

    def __init__(self, parent=None, width=5, height=4, dpi=100):
        super().__init__(parent=parent)

        layout = QVBoxLayout(self)
        self.fig = FigureCanvas(Figure(figsize=(width, height), dpi=dpi))
        layout.addWidget(self.fig)
        navbar = NavigationToolbar(self.fig, self)
        layout.addWidget(navbar)
        self.fig.figure.set_tight_layout(True)

        self.axes = self.fig.figure.add_subplot(111)



import sys

from PySide6.QtWidgets import QApplication

from orso_viewer.main_window import MainWindow


def main():
    app = QApplication(sys.argv)

    window = MainWindow()
    window.show()
    if len(sys.argv) > 1:
        window.read_file(sys.argv[1])

    sys.exit(app.exec())


if __name__ == "__main__":
    main()



import warnings

from traceback import format_exc

import yaml

from numpy import linspace
from orsopy.fileio import model_language as ml
from PySide6.QtCore import QObject, QRunnable, QSize, QThreadPool, Signal, Slot
from PySide6.QtWidgets import QErrorMessage, QFileDialog, QInputDialog, QMainWindow, QToolButton
from refnx.util import xray_energy

from orso_sample_builder.ui_main_window import Ui_MainWindow

DEFAULT_MODEL = """stack: vacuum | Fe 100 | Si
"""

try:
    import refnx
except ImportError:
    refnx = None


class WorkerSignals(QObject):
    resolved = Signal()
    finished = Signal()
    exception = Signal(str, str)


class SimulationWorker(QRunnable):
    """Thread to analyze a sample model in the background, can take a while to find e.g. materials."""

    def __init__(self, sample_model: ml.SampleModel, xray_energy=None):
        super(SimulationWorker, self).__init__()
        self.sample_model = sample_model
        if xray_energy is None:
            self.x = linspace(1e-10, 0.3, 300)
        else:
            self.x = linspace(1e-10, 0.5, 300)
        self.xray_energy = xray_energy
        self.ysim = None
        self.z = None
        self.SLDsim = None
        self.signals = WorkerSignals()

    @Slot()
    def run(self):
        # make a local copy not to modify the header information
        sample_model = ml.SampleModel.from_dict(self.sample_model.to_dict())

        with warnings.catch_warnings(action="error"):
            try:
                layers = sample_model.resolve_to_layers()
            except Exception as e:
                self.show_message("Could not resolve model to layers:", f"{format_exc()}")
                return
        self.signals.resolved.emit()

        if refnx is None:
            # refnx is not available, only resolve model
            return

        from refnx.reflect import SLD, ReflectModel, Structure

        if self.xray_energy is None:
            structure = Structure()
            for lj in layers:
                try:
                    m = SLD(lj.material.get_sld() * 1e6)
                except Exception as e:
                    self.show_message(f"Could not resolve material for layer {lj}:", f"{e}")
                    return
                structure |= m(lj.thickness.as_unit("angstrom"), lj.roughness.as_unit("angstrom"))
            model = ReflectModel(structure, bkg=0.0)
        else:
            structure = Structure()
            for lj in layers:
                try:
                    m = SLD(lj.material.get_sld(xray_energy=self.xray_energy / 1000.0) * 1e6)
                except Exception as e:
                    self.show_message(f"Could not resolve material for layer {lj}:", f"{e}")
                    return
                structure |= m(lj.thickness.as_unit("angstrom"), lj.roughness.as_unit("angstrom"))
            model = ReflectModel(structure, bkg=0.0)

        self.ysim = model(self.x)
        self.z, self.SLDsim = structure.sld_profile()

        self.signals.finished.emit()

    def show_message(self, message, traceback=None):
        self.signals.exception.emit(message, traceback)


class MainWindow(QMainWindow):
    current_model: ml.SampleModel

    def __init__(self):
        super(MainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.threadpool = QThreadPool()

        self.init_editor()
        self.activateWindow()

    def init_editor(self):
        doc = self.ui.model_editor.document()
        font = doc.defaultFont()
        font.setFamily("Courier New")
        doc.setDefaultFont(font)
        for chld in self.ui.editor_toolbar.children():
            if isinstance(chld, QToolButton):
                chld.pressed.connect(self.insert_class)
        from orsopy.fileio import model_language as ml

        try:
            for subcls in ml.SubStackType.__subclasses__():
                if subcls is ml.SubStack:
                    continue
                self.ui.complex_selector.addItem(subcls.__name__)
        except AttributeError:
            # feature not yet part of official orsopy release
            self.ui.complex_selector.hide()
        else:
            self.ui.complex_selector.currentIndexChanged.connect(self.insert_complex)

        # doc.setPlainText(DEFAULT_MODEL)
        doc.setPlainText("")
        self.current_model = ml.SampleModel(stack="vacuum | Fe 100 | Si")
        self.ui.model_editor.insert_class("SampleModel", **self.current_model.to_dict())

    def show_error(self, message, traceback=None):
        dia = QErrorMessage(parent=self)
        output = f"<h2>{message}</h2>"
        if traceback:
            output += f"<pre>{traceback}</pre>"
        dia.showMessage(output)
        dia.resize(QSize(600, 400))
        dia.setWindowTitle("Error: Sample Builder")
        dia.exec()

    def update_model(self):
        """
        Use the text in the editor to update the model. If an error occurs the editor
        shows a red background and the model stays unchanged.
        """
        sample_yaml = self.ui.model_editor.toPlainText()
        try:
            sample_dict = yaml.safe_load(sample_yaml)
        except Exception as e:
            self.ui.model_editor.setStyleSheet("background-color: #ffaaaa;")
            self.show_error("Issue in YAML code:", f"{e}")
            return
        else:
            self.ui.model_editor.setStyleSheet("background-color: #ffffff;")
            if "data_source" in sample_dict:
                sample_dict = sample_dict["data_source"]
            if "sample" in sample_dict:
                sample_dict = sample_dict["sample"]
            if "model" in sample_dict:
                sample_dict = sample_dict["model"]

        self.ui.model_viewer.preparing_sample()

        with warnings.catch_warnings(action="error"):
            try:
                sample_model = ml.SampleModel.from_dict(sample_dict)
            except Exception as e:
                self.show_error("Issue in SampleModel definition:", f"{format_exc()}")
                return
            else:
                self.current_model = sample_model
                # regenerate the model yaml
                self.ui.model_editor.setPlainText(self.current_model.to_yaml())

        if self.ui.radiation_xray.isChecked():
            xray_energy = self.ui.radiation_energy.value()
        else:
            xray_energy = None
        self._worker = SimulationWorker(self.current_model, xray_energy=xray_energy)
        self._worker.signals.resolved.connect(self.show_model)
        self._worker.signals.finished.connect(self.plot_model)
        self._worker.signals.exception.connect(self.show_error)
        self.threadpool.start(self._worker)

    def plot_model(self):
        self.ui.graph_refl.axes.clear()
        self.ui.graph_refl.axes.semilogy(self._worker.x, self._worker.ysim)
        self.ui.graph_refl.axes.set_xlabel("Q / Å$^{-1}$")
        self.ui.graph_refl.axes.set_ylabel("Reflectivity")
        self.ui.graph_refl.fig.draw_idle()
        self.ui.graph_sld.axes.clear()
        self.ui.graph_sld.axes.plot(self._worker.SLDsim, -self._worker.z)
        self.ui.graph_sld.axes.set_xlabel("SLD / 10$^{-6}$ Å$^{-2}$")
        self.ui.graph_sld.axes.set_ylabel("z-depth / Å")
        self.ui.graph_sld.fig.draw_idle()

    def show_model(self):
        self.ui.model_viewer.show_sample_model(self._worker.sample_model)

    def insert_class(self):
        self.ui.model_editor.insert_class(self.sender().text())

    def insert_complex(self, idx):
        if idx == 0:
            return
        self.ui.complex_selector.setCurrentIndex(0)
        self.ui.model_editor.insert_class(self.ui.complex_selector.itemText(idx))

    def load_model(self):
        sfile, sfilter = QFileDialog.getOpenFileName(self, filter="Sample Model YAML (*.yml *.yaml);;ORSO File (*.ort)")
        if not sfile:
            return
        if sfilter == "Sample Model YAML (*.yml *.yaml)":
            self.ui.model_editor.setPlainText(open(sfile, "r").read())
        else:
            from orsopy.fileio import load_orso

            datasets = load_orso(sfile)
            index = 0
            if len(datasets) > 1:
                dataset_names = [di.info.data_set for di in datasets]
                item, ok = QInputDialog.getItem(self, "Choose Dataset", "dataset", dataset_names, editable=False)
                if not ok:
                    return
                index = dataset_names.index(item)
            self.ui.model_editor.setPlainText(datasets[index].info.data_source.sample.model.to_yaml())
        self.update_model()

    def save_model(self):
        sfile, sfilter = QFileDialog.getSaveFileName(self, filter="Sample Model YAML (*.yml *.yaml);;ORSO File (*.ort)")
        if not sfile:
            return
        self.update_model()
        if sfilter == "Sample Model YAML (*.yml *.yaml)":
            text = self.ui.model_editor.toPlainText()
            open(sfile, "w").write(text)
        else:
            from orsopy.fileio import load_orso, save_orso

            datasets = load_orso(sfile)
            index = 0
            if len(datasets) > 1:
                dataset_names = [di.info.data_set for di in datasets]
                item, ok = QInputDialog.getItem(self, "Choose Dataset", "dataset", dataset_names, editable=False)
                if not ok:
                    return
                index = dataset_names.index(item)
            datasets[index].info.data_source.sample.model = self.current_model
            save_orso(datasets, sfile)

    def model_help(self):
        pass



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_window.ui'
##
## Created by: Qt User Interface Compiler version 6.9.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale, QMetaObject, QObject, QPoint, QRect, QSize, Qt,
                            QTime, QUrl)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient, QCursor, QFont, QFontDatabase, QGradient, QIcon,
                           QImage, QKeySequence, QLinearGradient, QPainter, QPalette, QPixmap, QRadialGradient,
                           QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QDockWidget, QDoubleSpinBox, QFrame, QGridLayout, QHBoxLayout,
                               QLabel, QMainWindow, QPushButton, QRadioButton, QSizePolicy, QSpacerItem, QSplitter,
                               QToolButton, QVBoxLayout, QWidget)

from orso_tools.mpl_canvas import MplCanvas
from orso_tools.sample_viewer import SampleViewer

from .model_editor import ModelEditor


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1256, 927)
        self.actionOpen = QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widget = QWidget(self.centralwidget)
        self.widget.setObjectName("widget")
        self.verticalLayout_6 = QVBoxLayout(self.widget)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.splitter = QSplitter(self.widget)
        self.splitter.setObjectName("splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.widget_2 = QWidget(self.splitter)
        self.widget_2.setObjectName("widget_2")
        self.verticalLayout = QVBoxLayout(self.widget_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget_3 = QWidget(self.widget_2)
        self.widget_3.setObjectName("widget_3")
        self.horizontalLayout_2 = QHBoxLayout(self.widget_3)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QLabel(self.widget_3)
        self.label.setObjectName("label")
        font = QFont()
        font.setFamilies(["Courier New"])
        font.setPointSize(10)
        self.label.setFont(font)

        self.horizontalLayout_2.addWidget(self.label)

        self.widget_5 = QWidget(self.widget_3)
        self.widget_5.setObjectName("widget_5")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget_5.sizePolicy().hasHeightForWidth())
        self.widget_5.setSizePolicy(sizePolicy)
        self.widget_5.setMinimumSize(QSize(170, 0))
        self.gridLayout = QGridLayout(self.widget_5)
        self.gridLayout.setSpacing(4)
        self.gridLayout.setObjectName("gridLayout")
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.label_2 = QLabel(self.widget_5)
        self.label_2.setObjectName("label_2")

        self.gridLayout.addWidget(self.label_2, 0, 0, 1, 1)

        self.radiation_neutron = QRadioButton(self.widget_5)
        self.radiation_neutron.setObjectName("radiation_neutron")
        self.radiation_neutron.setChecked(True)

        self.gridLayout.addWidget(self.radiation_neutron, 1, 0, 1, 1)

        self.radiation_xray = QRadioButton(self.widget_5)
        self.radiation_xray.setObjectName("radiation_xray")

        self.gridLayout.addWidget(self.radiation_xray, 2, 0, 1, 1)

        self.radiation_energy = QDoubleSpinBox(self.widget_5)
        self.radiation_energy.setObjectName("radiation_energy")
        self.radiation_energy.setDecimals(2)
        self.radiation_energy.setMaximum(99999.000000000000000)
        self.radiation_energy.setSingleStep(100.000000000000000)
        self.radiation_energy.setValue(8040.000000000000000)

        self.gridLayout.addWidget(self.radiation_energy, 2, 1, 1, 1)

        self.label_3 = QLabel(self.widget_5)
        self.label_3.setObjectName("label_3")

        self.gridLayout.addWidget(self.label_3, 2, 2, 1, 1)

        self.horizontalLayout_2.addWidget(self.widget_5)

        self.widget_4 = QWidget(self.widget_3)
        self.widget_4.setObjectName("widget_4")
        sizePolicy.setHeightForWidth(self.widget_4.sizePolicy().hasHeightForWidth())
        self.widget_4.setSizePolicy(sizePolicy)
        self.verticalLayout_7 = QVBoxLayout(self.widget_4)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.pushButton = QPushButton(self.widget_4)
        self.pushButton.setObjectName("pushButton")

        self.verticalLayout_7.addWidget(self.pushButton)

        self.pushButton_2 = QPushButton(self.widget_4)
        self.pushButton_2.setObjectName("pushButton_2")

        self.verticalLayout_7.addWidget(self.pushButton_2)

        self.pushButton_3 = QPushButton(self.widget_4)
        self.pushButton_3.setObjectName("pushButton_3")

        self.verticalLayout_7.addWidget(self.pushButton_3)

        self.horizontalLayout_2.addWidget(self.widget_4)

        self.verticalLayout.addWidget(self.widget_3)

        self.editor_toolbar = QFrame(self.widget_2)
        self.editor_toolbar.setObjectName("editor_toolbar")
        self.editor_toolbar.setFrameShape(QFrame.StyledPanel)
        self.editor_toolbar.setFrameShadow(QFrame.Sunken)
        self.horizontalLayout = QHBoxLayout(self.editor_toolbar)
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout.setContentsMargins(2, 2, 2, 2)
        self.button_layer = QToolButton(self.editor_toolbar)
        self.button_layer.setObjectName("button_layer")

        self.horizontalLayout.addWidget(self.button_layer)

        self.button_material = QToolButton(self.editor_toolbar)
        self.button_material.setObjectName("button_material")

        self.horizontalLayout.addWidget(self.button_material)

        self.button_composit = QToolButton(self.editor_toolbar)
        self.button_composit.setObjectName("button_composit")

        self.horizontalLayout.addWidget(self.button_composit)

        self.horizontalSpacer_2 = QSpacerItem(10, 20, QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)

        self.button_substack = QToolButton(self.editor_toolbar)
        self.button_substack.setObjectName("button_substack")

        self.horizontalLayout.addWidget(self.button_substack)

        self.complex_selector = QComboBox(self.editor_toolbar)
        self.complex_selector.addItem("")
        self.complex_selector.setObjectName("complex_selector")

        self.horizontalLayout.addWidget(self.complex_selector)

        self.horizontalSpacer_3 = QSpacerItem(10, 20, QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_3)

        self.toolButton = QToolButton(self.editor_toolbar)
        self.toolButton.setObjectName("toolButton")

        self.horizontalLayout.addWidget(self.toolButton)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.button_value = QToolButton(self.editor_toolbar)
        self.button_value.setObjectName("button_value")

        self.horizontalLayout.addWidget(self.button_value)

        self.button_complexvalue = QToolButton(self.editor_toolbar)
        self.button_complexvalue.setObjectName("button_complexvalue")

        self.horizontalLayout.addWidget(self.button_complexvalue)

        self.verticalLayout.addWidget(self.editor_toolbar)

        self.model_editor = ModelEditor(self.widget_2)
        self.model_editor.setObjectName("model_editor")
        self.model_editor.setAcceptRichText(False)

        self.verticalLayout.addWidget(self.model_editor)

        self.splitter.addWidget(self.widget_2)
        self.frame = QFrame(self.splitter)
        self.frame.setObjectName("frame")
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout_3 = QVBoxLayout(self.frame)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.model_viewer = SampleViewer(self.frame)
        self.model_viewer.setObjectName("model_viewer")

        self.verticalLayout_3.addWidget(self.model_viewer)

        self.splitter.addWidget(self.frame)

        self.verticalLayout_6.addWidget(self.splitter)

        self.update_button = QPushButton(self.widget)
        self.update_button.setObjectName("update_button")

        self.verticalLayout_6.addWidget(self.update_button)

        self.verticalLayout_2.addWidget(self.widget)

        MainWindow.setCentralWidget(self.centralwidget)
        self.dockWidget = QDockWidget(MainWindow)
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout_4 = QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.graph_refl = MplCanvas(self.dockWidgetContents)
        self.graph_refl.setObjectName("graph_refl")

        self.verticalLayout_4.addWidget(self.graph_refl)

        self.dockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(Qt.DockWidgetArea.RightDockWidgetArea, self.dockWidget)
        self.dockWidget_2 = QDockWidget(MainWindow)
        self.dockWidget_2.setObjectName("dockWidget_2")
        self.dockWidgetContents_2 = QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.verticalLayout_5 = QVBoxLayout(self.dockWidgetContents_2)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.graph_sld = MplCanvas(self.dockWidgetContents_2)
        self.graph_sld.setObjectName("graph_sld")

        self.verticalLayout_5.addWidget(self.graph_sld)

        self.dockWidget_2.setWidget(self.dockWidgetContents_2)
        MainWindow.addDockWidget(Qt.DockWidgetArea.RightDockWidgetArea, self.dockWidget_2)

        self.retranslateUi(MainWindow)
        self.update_button.pressed.connect(MainWindow.update_model)
        self.pushButton.pressed.connect(MainWindow.load_model)
        self.pushButton_2.pressed.connect(MainWindow.save_model)
        self.pushButton_3.pressed.connect(MainWindow.model_help)

        QMetaObject.connectSlotsByName(MainWindow)

    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", "MainWindow", None))
        self.actionOpen.setText(QCoreApplication.translate("MainWindow", "Open...", None))
        self.label.setText(
            QCoreApplication.translate("MainWindow", "data_source:\n" "   sample:\n" "     model:", None)
        )
        self.label_2.setText(QCoreApplication.translate("MainWindow", "Radiation:", None))
        self.radiation_neutron.setText(QCoreApplication.translate("MainWindow", "Neutron", None))
        self.radiation_xray.setText(QCoreApplication.translate("MainWindow", "X-Ray", None))
        self.label_3.setText(QCoreApplication.translate("MainWindow", "eV", None))
        self.pushButton.setText(QCoreApplication.translate("MainWindow", "Load Model...", None))
        self.pushButton_2.setText(QCoreApplication.translate("MainWindow", "Save Model...", None))
        self.pushButton_3.setText(QCoreApplication.translate("MainWindow", "Model Help...", None))
        self.button_layer.setText(QCoreApplication.translate("MainWindow", "Layer", None))
        self.button_material.setText(QCoreApplication.translate("MainWindow", "Material", None))
        self.button_composit.setText(QCoreApplication.translate("MainWindow", "Composit", None))
        self.button_substack.setText(QCoreApplication.translate("MainWindow", "SubStack", None))
        self.complex_selector.setItemText(0, QCoreApplication.translate("MainWindow", "Complex Item...", None))

        self.toolButton.setText(QCoreApplication.translate("MainWindow", "ModelParameters", None))
        self.button_value.setText(QCoreApplication.translate("MainWindow", "Value", None))
        self.button_complexvalue.setText(QCoreApplication.translate("MainWindow", "ComplexValue", None))
        self.update_button.setText(QCoreApplication.translate("MainWindow", "Update Model", None))
        self.dockWidget.setWindowTitle(QCoreApplication.translate("MainWindow", "Reflectivity", None))
        self.dockWidget_2.setWindowTitle(QCoreApplication.translate("MainWindow", "SLD Profile", None))

    # retranslateUi



from dataclasses import fields

from PySide6.QtCore import QRegularExpression, Qt
from PySide6.QtGui import QColor, QFont, QSyntaxHighlighter, QTextCharFormat, QTextCursor
from PySide6.QtWidgets import QTextEdit


class OrsoSampleHighlighter(QSyntaxHighlighter):
    _orso_special = ["stack"]

    def __init__(self, parent=None):
        super(OrsoSampleHighlighter, self).__init__(parent)
        self.dict_key_format = QTextCharFormat()
        self.dict_key_format.setFontWeight(QFont.Bold)
        self.dict_key_format.setForeground(Qt.GlobalColor.darkMagenta)
        self.special_key_format = QTextCharFormat()
        self.special_key_format.setFontWeight(QFont.Bold)
        self.special_key_format.setForeground(Qt.GlobalColor.darkGreen)
        self.stack_format = QTextCharFormat()
        self.stack_format.setFontWeight(QFont.Bold)
        self.stack_format.setForeground(Qt.GlobalColor.black)
        self.comment_format = QTextCharFormat()
        self.comment_format.setForeground(Qt.GlobalColor.gray)
        self.inline_dict_format = QTextCharFormat()
        self.inline_dict_format.setBackground(QColor(240, 240, 255))

    def highlightBlock(self, text):
        expression = QRegularExpression("\\b[a-zA-Z_0-9]*:")
        i = expression.globalMatch(text)
        while i.hasNext():
            match = i.next()
            name = match.capturedTexts()[0][:-1]
            if name in self._orso_special:
                self.setFormat(match.capturedStart(), match.capturedLength(), self.special_key_format)
            else:
                self.setFormat(match.capturedStart(), match.capturedLength(), self.dict_key_format)

        expression = QRegularExpression("(?<=stack:).*")
        i = expression.globalMatch(text)
        while i.hasNext():
            match = i.next()
            self.setFormat(match.capturedStart(), match.capturedLength(), self.stack_format)

        expression = QRegularExpression("# .*$")
        i = expression.globalMatch(text)
        while i.hasNext():
            match = i.next()
            self.setFormat(match.capturedStart(), match.capturedLength(), self.comment_format)

        expression = QRegularExpression("\\{.*\\}")
        i = expression.globalMatch(text)
        while i.hasNext():
            match = i.next()
            self.setFormat(match.capturedStart(), match.capturedLength(), self.inline_dict_format)


class ModelEditor(QTextEdit):

    def __init__(self, parent):
        QTextEdit.__init__(self, parent)
        self.highlighter = OrsoSampleHighlighter(self.document())

    def keyPressEvent(self, event):
        tab_char = "  "  # could be anything including spaces
        if event.key() == Qt.Key_Backtab:
            # get current cursor
            cur = self.textCursor()
            cur.clearSelection()

            # move to begining of line and select text to first word
            cur.movePosition(QTextCursor.StartOfLine)
            cur.movePosition(QTextCursor.NextWord, QTextCursor.KeepAnchor)
            sel_text = cur.selectedText()

            # if the text starts with the tab_char, replace it
            if sel_text.startswith(tab_char):
                text = sel_text.replace(tab_char, "", 1)
                cur.insertText(text)
        elif event.key() == Qt.Key_Tab:
            # get current cursor
            cur = self.textCursor()
            cur.clearSelection()

            # move to begining of line and select text to first word
            cur.movePosition(QTextCursor.StartOfLine)
            cur.insertText(tab_char)
            cur.movePosition(QTextCursor.NextWord, QTextCursor.KeepAnchor)

        else:
            return QTextEdit.keyPressEvent(self, event)

    def insert_class(self, cls_name, **initial_values):
        from orsopy.fileio import model_language as ml

        try:
            from orsopy.fileio import model_complex as mc
        except ImportError:
            mc = ml

        namespace = {**ml.__dict__, **mc.__dict__}

        cls = namespace[cls_name]

        c = self.textCursor()
        pre_c_line = self.toPlainText()[: int(c.anchor())].rsplit("\n", 1)[-1]
        expression = QRegularExpression("^ *")
        i = expression.globalMatch(pre_c_line)
        if i.hasNext():
            indent = " " * i.next().capturedLength()
        else:
            indent = ""
        if pre_c_line.strip().endswith(":"):
            insert_txt = "\n"
        else:
            insert_txt = ""

        if cls_name == "Value":
            insert_txt = "{magnitude: 0., unit: ''}"
        elif cls_name == "ComplexValue":
            insert_txt = "{real: 0., imag: 0., unit: ''}"
        else:
            for field in fields(cls):
                if field.name in ["comment"]:
                    continue
                type_str = f"{field.type}"
                type_str = type_str.replace("typing.", "")
                type_str = type_str.replace("orsopy.fileio.base.", "")
                type_str = type_str.replace("orsopy.fileio.model_language.", "")
                type_str = type_str.replace("orsopy.fileio.model_building_blocks.", "")
                type_str = type_str.replace("orsopy.fileio.model_complex.", "")
                if field.name in initial_values:
                    insert_txt += f"{field.name}: {initial_values[field.name]}\n"
                elif field.name in ["sub_stack_class"]:
                    insert_txt += f"{field.name}: {field.default}\n"
                else:
                    insert_txt += f"{field.name}: null # {type_str}\n"
        if cls_name != "SampleModel":
            insert_txt = insert_txt.replace("\n", "\n  " + indent)
        c.insertText(insert_txt)



import os

from orsopy.fileio import ComplexValue, Value
from orsopy.fileio import model_language as ml
from PySide6 import QtCore, QtSvgWidgets


class SampleViewer(QtSvgWidgets.QSvgWidget):
    def __init__(self, parent=None):
        super(SampleViewer, self).__init__(parent)

    COLORS = ["#ffaaaa", "#aaffaa", "#aaaaff", "#ff00ff", "#00ffff"]

    def no_sample(self):
        svg = """<svg width="100" height="100" viewbox="0 0 100 100" xmlns="http://www.w3.org/2000/svg">
                <text x="50" y="8" fill="black" font-size="6" text-anchor="middle">No sample model in dataset.</text>
        </svg>"""
        self.renderer().load(QtCore.QByteArray(svg))
        self.renderer().setAspectRatioMode(QtCore.Qt.AspectRatioMode.KeepAspectRatio)

    def preparing_sample(self):
        svg = """<svg width="100" height="100" viewbox="0 0 100 100" xmlns="http://www.w3.org/2000/svg">
                <text x="50" y="8" fill="black" font-size="6" text-anchor="middle">Evaluating model...</text>
        </svg>"""
        self.renderer().load(QtCore.QByteArray(svg))
        self.renderer().setAspectRatioMode(QtCore.Qt.AspectRatioMode.KeepAspectRatio)

    def show_sample_model(self, model: "ml.SampleModel"):
        items = model.resolve_stack()

        pos, height, svg_inside = self.build_substack(10, 10, 0, items)

        svg = f"""<svg width="100" height="{height}" viewbox="0 0 {height} 100" xmlns="http://www.w3.org/2000/svg">
        <text x="50" y="8" fill="black" font-size="6" text-anchor="middle">Sample Model:</text>
        {svg_inside}
</svg>"""
        self.renderer().load(QtCore.QByteArray(svg))
        self.renderer().setAspectRatioMode(QtCore.Qt.AspectRatioMode.KeepAspectRatio)

    def build_substack(self, pos, height, indent, items):
        svg_inside = ""
        for item in items:
            if isinstance(item, ml.Layer):
                width = 100 - 5 - indent * 1.2
                svg_inside += f"""
    <rect width="{width}" height="18" x="{indent + 5}" y="{pos + 1}" rx="2" ry="2" fill="#cccccc" />
    <text x="{indent + 10}" y="{pos + 8}" fill="black" font-size="4">Layer {item.original_name or ''}</text>
    <text x="{indent + 20}" y="{pos + 13}" fill="black" font-size="3">
        thickness={item.thickness.magnitude} {item.thickness.unit}
        | roughness={item.roughness.magnitude} {item.roughness.unit}
    </text>"""
                pos += 20
                height += 20
            elif isinstance(item, ml.SubStack):
                new_pos, new_height, svg_substack = self.build_substack(pos + 7, height + 8, indent + 5, item.sequence)
                width = 100 - 5 - indent * 1.1
                pre_str = ""
                post_str = ""
                if item.repetitions != 1:
                    pre_str = f"{item.repetitions} X "
                if getattr(item, "environment", None) is not None:
                    post_str = f" in {item.environment.original_name or repr(item.environment)}"

                svg_inside += f"""
    <rect width="{width}" height="{new_height - height}" x="{indent + 5}" y="{pos}" rx="2" ry="2"
        fill="{self.COLORS[(indent // 5) % len(self.COLORS)]}" />
    <text x="{indent + 10}" y="{pos + 6}" fill="black" font-size="4">
        {pre_str}SubStack {item.original_name or ''}{post_str}
    </text>"""

                svg_inside += svg_substack
                pos = new_pos + 1
                height = new_height
            elif isinstance(item, ml.SubStackType):
                width = 100 - 5 - indent * 1.2
                parameter_text = ""
                param_height = 0
                for param, value in item.to_dict().items():
                    if param in ["sub_stack_class", "original_name"]:
                        continue
                    obj = getattr(item, param)
                    value_str = repr(obj)
                    if isinstance(obj, Value):
                        value_str = f"{obj.magnitude} {obj.unit}"
                    elif isinstance(obj, ComplexValue):
                        value_str = f"{obj.real}+{obj.imag}i {obj.unit}"
                    elif isinstance(obj, ml.Material):
                        value_str = getattr(obj, "original_name", obj.formula)
                    parameter_text += f"""
    <text x="{indent + 12}" y="{pos + 13 + param_height}" fill="black" font-size="3">{param}={value_str}</text>"""
                    param_height += 3
                svg_inside += f"""
    <rect width="{width}" height="{10 + param_height}" x="{indent + 5}" y="{pos + 1}" rx="2" ry="2" fill="#ffffaa" />
    <text x="{indent + 10}" y="{pos + 8}" fill="black" font-size="4">
        {item.__class__.__name__} {item.original_name or ''}
    </text>"""
                svg_inside += parameter_text
                pos += 12 + param_height
                height += 12 + param_height

        return pos, height, svg_inside



from orsopy import fileio
from orsopy.fileio import model_language
from PySide6.QtCore import QObject, QRunnable, Qt, QThreadPool, Signal, Slot
from PySide6.QtWidgets import QMainWindow, QStyle, QTreeWidgetItem

from orso_viewer.ui_main_window import Ui_MainWindow

try:
    import refnx
except ImportError:
    refnx = None


class WorkerSignals(QObject):
    resolved = Signal()
    finished = Signal()


class SimulationWorker(QRunnable):
    """Thread to analyze a sample model in the background, can take a while to find e.g. materials."""

    def __init__(self, sample_model: model_language.SampleModel, x, xray_energy=None):
        super(SimulationWorker, self).__init__()
        self.sample_model = sample_model
        self.x = x
        self.xray_energy = xray_energy
        self.ysim = None
        self.signals = WorkerSignals()

    @Slot()
    def run(self):
        # make a local copy not to modify the header information
        sample_model = model_language.SampleModel.from_dict(self.sample_model.to_dict())

        layers = sample_model.resolve_to_layers()
        self.signals.resolved.emit()

        if refnx is None:
            # refnx is not available, only resolve model
            return

        from refnx.reflect import SLD, ReflectModel, Structure

        if self.xray_energy is not None:
            structure = Structure()
            for lj in layers:
                m = SLD(lj.material.get_sld() * 1e6)
                structure |= m(lj.thickness.as_unit("angstrom"), lj.roughness.as_unit("angstrom"))
            model = ReflectModel(structure, bkg=0.0)
        else:
            structure = Structure()
            for lj in layers:
                m = SLD(lj.material.get_sld(xray_energy=self.xray_energy) * 1e6)
                structure |= m(lj.thickness.as_unit("angstrom"), lj.roughness.as_unit("angstrom"))
            model = ReflectModel(structure, bkg=0.0)

        self.ysim = model(self.x)

        self.signals.finished.emit()


class MainWindow(QMainWindow):
    datasets: list[fileio.OrsoDataset]

    def __init__(self):
        super(MainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.datasets = []
        self.threadpool = QThreadPool()

        self.ui.header_tree.setColumnCount(2)
        self.ui.header_tree.setHeaderLabels(["Key", "Type"])

        doc = self.ui.header_data.document()
        font = doc.defaultFont()
        font.setFamily("Courier New")
        doc.setDefaultFont(font)

        self.ui.button_expand.setIcon(self.style().standardIcon(QStyle.SP_TitleBarUnshadeButton))
        self.ui.button_collapse.setIcon(self.style().standardIcon(QStyle.SP_MediaPlay))

    def read_file(self, filename):
        self.datasets = fileio.load_orso(filename)

        self.ui.dataset_list.clear()
        self.ui.dataset_list.addItems([str(di.info.data_set) for di in self.datasets])

        self.ui.dataset_list.setCurrentRow(0)

    @Slot(int)
    def dataset_selected(self, index):
        self.plot_dataset(index)
        self.update_header(index)

    def plot_dataset(self, index):
        dataset = self.datasets[index]
        sc = self.ui.data_plot
        sc.axes.clear()
        sc.axes.errorbar(dataset.data[:, 0], dataset.data[:, 1], dataset.data[:, 2], label="data")
        sc.axes.set_yscale("log")
        sc.axes.set_xlabel(dataset.info.columns[0].name + " / " + (dataset.info.columns[0].unit or ""))
        sc.axes.set_ylabel(dataset.info.columns[1].name + " / " + (dataset.info.columns[1].unit or "1"))
        sc.fig.draw_idle()

        try:
            sample_model = dataset.info.data_source.sample.model
        except AttributeError:
            self.ui.sample_viewer.no_sample()
        else:
            self.ui.sample_viewer.preparing_sample()
            self.worker = SimulationWorker(sample_model, dataset.data[:, 0])
            self.worker.signals.resolved.connect(self.show_sample)
            self.worker.signals.finished.connect(self.plot_simulation)
            self.threadpool.start(self.worker)

    @Slot()
    def show_sample(self):
        # the worker already resolved all layers of the model
        self.ui.sample_viewer.show_sample_model(self.worker.sample_model)

    @Slot()
    def plot_simulation(self):
        x = self.worker.x
        ysim = self.worker.ysim
        sc = self.ui.data_plot
        sc.axes.semilogy(x, ysim, label="model")
        sc.axes.legend()
        sc.fig.draw_idle()

    def build_subtree(self, parent, data, obj):
        for key, value in data.items():
            if isinstance(obj, list):
                subobj = obj[key]
                key = str(key)
            else:
                subobj = getattr(obj, key, None)
            if isinstance(subobj, fileio.Header):
                subitem = QTreeWidgetItem([key, subobj.__class__.__name__])
                subitem.setData(0, Qt.ItemDataRole.UserRole, subobj.to_yaml())
                parent.addChild(subitem)
                self.build_subtree(subitem, value, subobj)
            elif isinstance(subobj, list):
                subitem = QTreeWidgetItem([key, "List"])
                subitem.setData(0, Qt.ItemDataRole.UserRole, str(subobj))
                parent.addChild(subitem)
                self.build_subtree(subitem, dict(enumerate(value)), subobj)
            else:
                subitem = QTreeWidgetItem([key, str(subobj)])
                parent.addChild(subitem)

    def update_header(self, index):
        tv = self.ui.header_tree
        dataset: fileio.OrsoDataset = self.datasets[index]
        data = dataset.info.to_dict()
        items = []
        for key, value in data.items():
            obj = getattr(dataset.info, key)
            if isinstance(obj, fileio.Header):
                item = QTreeWidgetItem([key, obj.__class__.__name__])
                item.setData(0, Qt.ItemDataRole.UserRole, obj.to_yaml())
                self.build_subtree(item, value, obj)
            elif isinstance(obj, list):
                item = QTreeWidgetItem([key, "List"])
                item.setData(0, Qt.ItemDataRole.UserRole, str(obj))
                self.build_subtree(item, dict(enumerate(value)), obj)
            else:
                item = QTreeWidgetItem([key, str(obj)])
            items.append(item)

        tv.clear()
        tv.insertTopLevelItems(0, items)

    def show_dataset_item(self, item: QTreeWidgetItem, idx):
        data = item.data(0, Qt.ItemDataRole.UserRole)
        if data is None:
            self.ui.header_data.setText(item.text(1))
        else:
            self.ui.header_data.setText(data)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_window.ui'
##
## Created by: Qt User Interface Compiler version 6.9.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale, QMetaObject, QObject, QPoint, QRect, QSize, Qt,
                            QTime, QUrl)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient, QCursor, QFont, QFontDatabase, QGradient, QIcon,
                           QImage, QKeySequence, QLinearGradient, QPainter, QPalette, QPixmap, QRadialGradient,
                           QTransform)
from PySide6.QtWidgets import (QApplication, QHBoxLayout, QHeaderView, QListWidget, QListWidgetItem, QMainWindow, QMenu,
                               QMenuBar, QSizePolicy, QSpacerItem, QSplitter, QStatusBar, QTabWidget, QTextBrowser,
                               QToolButton, QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget)

from orso_tools.mpl_canvas import MplCanvas
from orso_tools.sample_viewer import SampleViewer


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1038, 728)
        self.actionOpen = QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_2 = QVBoxLayout(self.centralwidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QSplitter(self.centralwidget)
        self.splitter.setObjectName("splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.dataset_list = QListWidget(self.splitter)
        self.dataset_list.setObjectName("dataset_list")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dataset_list.sizePolicy().hasHeightForWidth())
        self.dataset_list.setSizePolicy(sizePolicy)
        self.dataset_list.setMinimumSize(QSize(120, 0))
        self.splitter.addWidget(self.dataset_list)
        self.tabWidget = QTabWidget(self.splitter)
        self.tabWidget.setObjectName("tabWidget")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy1.setHorizontalStretch(10)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy1)
        self.plot_tab = QWidget()
        self.plot_tab.setObjectName("plot_tab")
        self.verticalLayout = QVBoxLayout(self.plot_tab)
        self.verticalLayout.setObjectName("verticalLayout")
        self.data_plot = MplCanvas(self.plot_tab)
        self.data_plot.setObjectName("data_plot")

        self.verticalLayout.addWidget(self.data_plot)

        self.tabWidget.addTab(self.plot_tab, "")
        self.header_tab = QWidget()
        self.header_tab.setObjectName("header_tab")
        self.verticalLayout_4 = QVBoxLayout(self.header_tab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.splitter_2 = QSplitter(self.header_tab)
        self.splitter_2.setObjectName("splitter_2")
        self.splitter_2.setOrientation(Qt.Horizontal)
        self.verticalLayoutWidget = QWidget(self.splitter_2)
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout_3 = QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.button_expand = QToolButton(self.verticalLayoutWidget)
        self.button_expand.setObjectName("button_expand")
        self.button_expand.setIconSize(QSize(24, 16))

        self.horizontalLayout.addWidget(self.button_expand)

        self.button_collapse = QToolButton(self.verticalLayoutWidget)
        self.button_collapse.setObjectName("button_collapse")

        self.horizontalLayout.addWidget(self.button_collapse)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.header_tree = QTreeWidget(self.verticalLayoutWidget)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(1, "2")
        __qtreewidgetitem.setText(0, "1")
        self.header_tree.setHeaderItem(__qtreewidgetitem)
        self.header_tree.setObjectName("header_tree")
        self.header_tree.setColumnCount(2)
        self.header_tree.header().setDefaultSectionSize(200)

        self.verticalLayout_3.addWidget(self.header_tree)

        self.splitter_2.addWidget(self.verticalLayoutWidget)
        self.header_data = QTextBrowser(self.splitter_2)
        self.header_data.setObjectName("header_data")
        self.header_data.setTextInteractionFlags(
            Qt.LinksAccessibleByKeyboard
            | Qt.LinksAccessibleByMouse
            | Qt.TextBrowserInteraction
            | Qt.TextSelectableByKeyboard
            | Qt.TextSelectableByMouse
        )
        self.splitter_2.addWidget(self.header_data)

        self.verticalLayout_4.addWidget(self.splitter_2)

        self.tabWidget.addTab(self.header_tab, "")
        self.sample_tab = QWidget()
        self.sample_tab.setObjectName("sample_tab")
        self.horizontalLayout_2 = QHBoxLayout(self.sample_tab)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.sample_viewer = SampleViewer(self.sample_tab)
        self.sample_viewer.setObjectName("sample_viewer")

        self.horizontalLayout_2.addWidget(self.sample_viewer)

        self.tabWidget.addTab(self.sample_tab, "")
        self.splitter.addWidget(self.tabWidget)

        self.verticalLayout_2.addWidget(self.splitter)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName("menubar")
        self.menubar.setGeometry(QRect(0, 0, 1038, 21))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionOpen)

        self.retranslateUi(MainWindow)
        self.dataset_list.currentRowChanged.connect(MainWindow.dataset_selected)
        self.header_tree.itemClicked.connect(MainWindow.show_dataset_item)
        self.button_expand.pressed.connect(self.header_tree.expandAll)
        self.button_collapse.pressed.connect(self.header_tree.collapseAll)

        self.tabWidget.setCurrentIndex(0)

        QMetaObject.connectSlotsByName(MainWindow)

    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", "MainWindow", None))
        self.actionOpen.setText(QCoreApplication.translate("MainWindow", "Open...", None))
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.plot_tab), QCoreApplication.translate("MainWindow", "Data", None)
        )
        self.button_expand.setText(QCoreApplication.translate("MainWindow", "...", None))
        self.button_collapse.setText(QCoreApplication.translate("MainWindow", "...", None))
        self.header_data.setHtml(
            QCoreApplication.translate(
                "MainWindow",
                '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
                '<p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">Select ORSO type line to show data in YAML format.</p></body></html>',
                None,
            )
        )
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.header_tab),
            QCoreApplication.translate("MainWindow", "Header Information", None),
        )
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.sample_tab), QCoreApplication.translate("MainWindow", "Sample", None)
        )
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", "File", None))

    # retranslateUi
